/*
 * AmbientLightMonitor_Defs.h
 *
 *  Created on: Jul 25, 2015
 *      Author: RK
 */

#ifndef AMBIENTLIGHTMONITOR_DEFS_H_
#define AMBIENTLIGHTMONITOR_DEFS_H_

#define BIT7 0x80
#define BIT6 0x40
#define BIT5 0x20
#define BIT4 0x10
#define BIT3 0x08
#define BIT2 0x04
#define BIT1 0x02
#define BIT0 0x01

#define HIGH 1
#define LOW 0

#define TRUE 1
#define FALSE 0

#define ROW 0
#define COL 1

//for displaying measurements
//----------------------------------------------------------------------
#define DIGIT_HEIGHT_PIX 30
#define DIGIT_WIDTH_PIX 24
#define DIGIT_WIDTH_BYTE (DIGIT_WIDTH_PIX/8)
#define UNIT_HEIGHT_PIX 30
#define UNIT_WIDTH_PIX 32
#define UNIT_WIDTH_BYTE (UNIT_WIDTH_PIX/8)
#define DIGIT_PIXDATA_ARRAY_LEN (DIGIT_HEIGHT_PIX*DIGIT_WIDTH_BYTE)
#define UNIT_PIXDATA_ARRAY_LEN (UNIT_HEIGHT_PIX*UNIT_WIDTH_BYTE)
#define DIGIT_BEGIN_LINE ((128/2)-(DIGIT_HEIGHT_PIX/2))
#define UNIT_BEGIN_LINE ((128/2)-(UNIT_HEIGHT_PIX/2))
#define DIGIT_POS_1 1
#define DIGIT_POS_2 2
#define DIGIT_POS_3 3
#define UNIT_MICROAMPS 1
#define UNIT_MILLIAMPS 2
#define UNIT_MILLIVOLTS 3
#define UNIT_VOLTS 4
#define UNIT_HERTZ 5
#define VOLTAGE 0
#define CURRENT 1
#define FREQUENCY 2
#define DEC_POINT_OFF 0
#define DEC_POINT_ON 1
#define DEC_POINT_HEIGHT 4
#define DEC_PNT_LINE_BEGIN (DIGIT_BEGIN_LINE + DIGIT_HEIGHT_PIX) - DEC_POINT_HEIGHT
#define DEC_PNT_LINE_END (DIGIT_BEGIN_LINE + DIGIT_HEIGHT_PIX)
//----------------------------------------------------------------------

#define NUM_LINE_DATA_BYTES 16

#define IDLE 0
#define DATA_UPDATE_MODE_SENT 1
#define FIRST_LINE_ADDR_SENT 2
#define LINE_DATA_SENT 3
#define PRE_ADDR_DUMMY_SENT 4
#define NEW_ADDR_SENT 5
#define LAST_LINE_DATA_SENT 6
#define FIRST_POST_DATA_DUMMY_SENT 7
#define SECOND_POST_DATA_DUMMY_SENT 8
#define ALL_CLEAR_SENT 9
#define FIRST_ALLCLEAR_DUMMY_SENT 10
#define SECOND_ALLCLEAR_DUMMY_SENT 11

//global variables shared across files
extern code unsigned char LCDLineAddr_LUT[];
extern unsigned char LCDTxState;
extern xdata unsigned char LCDDisplayData[DIGIT_HEIGHT_PIX][NUM_LINE_DATA_BYTES];
extern unsigned char LCDLineStart;
extern unsigned char LCDLineStop;
extern bit ADC_CONV_COMPLETE;
extern unsigned int RisingEdgeCount;
extern unsigned int FallingEdgeCount;

//function prototypes
void UpdateLCD();
void LCDAllClear();
void InitLCD();
void SetLCDDigit(unsigned char, unsigned char*);
void SetLCDUnit(unsigned char);
void SetDecimalPoint(unsigned char);
void ConvertMeasurementandDisplay(unsigned char, unsigned int);
unsigned char* MatchDigittoArray(unsigned char);

//port pins
sbit RGBLED_G = P1^4;
sbit RGBLED_B = P1^5;
sbit RGBLED_R = P1^6;

sbit SPI_CS = P0^1;

extern code unsigned char LCD_Digit0[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Digit1[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Digit2[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Digit3[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Digit4[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Digit5[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Digit6[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Digit7[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Digit8[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Digit9[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Error[DIGIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_microAmps[UNIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_milliAmps[UNIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_milliVolts[UNIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Volts[UNIT_PIXDATA_ARRAY_LEN];
extern code unsigned char LCD_Hz[UNIT_PIXDATA_ARRAY_LEN];

#define ADC0_PAGE    0x00 // ADC SFR Page
#define ADCMUXIN_P1_1 0x09
#define ADCMUXIN_GND 0x13
#define ADC_2POWER10 1024
#define ADC_VREF_MILLIVOLTS 2400
#define START_CONV 1
#define TWO_POWER_5 32

#endif /* AMBIENTLIGHTMONITOR_DEFS_H_ */
