
//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <SI_EFM8UB1_Register_Enums.h>                  // SFR declarations
#include "InitDevice.h"
#include "AmbientLightMonitor_Defs.h"




//-----------------------------------------------------------------------------
// main() Routine
// ----------------------------------------------------------------------------
#pragma optimize(8, SPEED)
int main (void)
{
	unsigned int RawADCResult, ADCMeasurement;
	float ADCFactor;
	unsigned char NumberofMeasurements;

	// Call hardware initialization routine
	enter_DefaultMode_from_RESET();

	//clear the LCD screen to all white
	InitLCD();

	// Enable global interrupts
	IE_EA = 1;

	ADCFactor = (float)ADC_VREF_MILLIVOLTS/ADC_2POWER10;

	SFRPAGE = ADC0_PAGE;
	ADC0MX = ADCMUXIN_P1_1;

	NumberofMeasurements = 0;
	RawADCResult = 0;

	RisingEdgeCount = 0;
	FallingEdgeCount = 0;

	while(1)
	{
			ADC0CN0_ADBUSY = START_CONV;

			//wait until the conversion is complete
			while(ADC_CONV_COMPLETE == FALSE);
			ADC_CONV_COMPLETE = FALSE;

			//retrieve the 10-bit ADC value and add it to the accumulating sum in RawADCResult
			SFRPAGE = ADC0_PAGE;
			RawADCResult = RawADCResult + ADC0;
			NumberofMeasurements++;

			/*if we have enough measurements to compute an average,
			 shift right to divide the sum by the number of measurements*/
			if(NumberofMeasurements == TWO_POWER_5)
			{
				RawADCResult = RawADCResult >> 5;
				NumberofMeasurements = 0;

				//convert the averaged conversion result to a current measurement and display
				//the actual value of the resistor in the test circuit is 4.6 kOhms
				ADCMeasurement = (RawADCResult*ADCFactor)/4.6;
				//ConvertMeasurementandDisplay(CURRENT, ADCMeasurement);

				//display the number of cycles counted in the last second
				ConvertMeasurementandDisplay(FREQUENCY, RisingEdgeCount);

				RisingEdgeCount = 0;
				FallingEdgeCount = 0;
			}

			//the value 10700 (rather than 10000) was empirically found to produce a delay closer to one second
			SFRPAGE = TIMER3_PAGE; TMR3 = 0; while(TMR3 < (10700/TWO_POWER_5));
	}
}
