/*
 * LCDControl.c
 *
 *  Created on: Jul 8, 2015
 *      Author: RK
 */

#include <SI_EFM8UB1_Register_Enums.h>		// SFR declarations
#include "AmbientLightMonitor_Defs.h"


bit DecimalPointOnorOff;

//global variables
unsigned char LCDModeSelect;
unsigned char LCDTxState = IDLE;
xdata unsigned char  LCDDisplayData[DIGIT_HEIGHT_PIX][NUM_LINE_DATA_BYTES];




///////////////////////////////////////////////////////////////////////////////////////

void InitLCD()
{
	unsigned char row, column_byte;

	SPI_CS = LOW;															//make sure that the chip select signal is LOW

	for(row = 0; row < DIGIT_HEIGHT_PIX; row++)
	{
		for(column_byte = 0; column_byte < NUM_LINE_DATA_BYTES; column_byte++)
		{
			LCDDisplayData[row][column_byte] = 0xFF;
		}
	}

	LCDAllClear();										//first clear the LCD to all white pixels
}

void UpdateLCD()
{
	SPI_CS = HIGH;																//set chip select HIGH to enable data transfer to the LCD

	SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 12);				//this ensures adequate delay from driving chip select high to the start of data transfer (tsSCS in the LCD data sheet)

	LCDTxState = DATA_UPDATE_MODE_SENT;								//this tells the ISR to send the address byte after this mode select byte is sent

	SFRPAGE = SPI0_PAGE;
	SPI0DAT = 0x00 | BIT7;														//this initiates transfer of the first byte (the mode select byte)
																										// 0x00 | BIT7 corresponds to "data update mode"

	/*when transfer of this byte is complete, the SPI interrupt will fire, and the rest
	of the transfer is handled in the ISR*/
}


void LCDAllClear()
{
	unsigned char row, column_byte;

	for(row = 0; row < DIGIT_HEIGHT_PIX; row++)
	{
		for(column_byte = 0; column_byte < NUM_LINE_DATA_BYTES; column_byte++)
		{
			LCDDisplayData[row][column_byte] = 0xFF;
		}
	}

	SPI_CS = HIGH;																//Set chip select HIGH to enable data transfer to the LCD

	SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 12);			//this ensures adequate delay from driving chip select high to the start of data transfer (tsSCS in the LCD data sheet)

	LCDTxState = ALL_CLEAR_SENT;										//this tells the ISR that this transfer sequence is an all clear command, not a transmission of pixel data

	SFRPAGE = SPI0_PAGE;
	SPI0DAT = 0x00 | BIT5;														//this initiates transfer of the first byte (the mode select byte)
																										// 0x00 | BIT5 corresponds to "all clear mode"

	/*when transfer of this byte is complete, the SPI interrupt will fire, and the rest
	of the transfer is handled in the ISR*/
}

void SetLCDDigit(unsigned char DigitPosition, unsigned char *LCD_Digit)
{
	unsigned char row;
	unsigned char column_byte;
	unsigned char column_byte_begin, column_byte_end;

	/*this switch statement determines which column bytes to
	 modify based on the chosen digit position (first, second, or third)*/
	switch(DigitPosition)
	{
		case DIGIT_POS_1: column_byte_begin = 0;
		break;

		case DIGIT_POS_2: column_byte_begin = DIGIT_WIDTH_BYTE;
		break;

		case DIGIT_POS_3: column_byte_begin = DIGIT_WIDTH_BYTE*2;
		break;
	}

	column_byte_end = column_byte_begin + DIGIT_WIDTH_BYTE;

	/*here the LCD display data array is loaded with the bytes
	 from the appropriate pixel data array generated by Scilab*/
	for(row = 0; row < DIGIT_HEIGHT_PIX; row++)
	{
		for(column_byte = column_byte_begin; column_byte < column_byte_end; column_byte++)
		{
			LCDDisplayData[row][column_byte] = *LCD_Digit;
			LCD_Digit++;
		}
	}

	//wait until the SPI state variable indicates that the bus is available for a new transfer
	while(LCDTxState != IDLE);

	//the SPI state machine needs to know the first and last lines to be updated
	LCDLineStart = DIGIT_BEGIN_LINE;
	LCDLineStop = DIGIT_BEGIN_LINE + DIGIT_HEIGHT_PIX;
	UpdateLCD();
}

void SetLCDUnit(unsigned char SelectedUnit)
{
	unsigned char row;
	unsigned char column_byte;
	unsigned char column_byte_begin, column_byte_end;
	unsigned char* LCD_Unit;

	//here the "+ 1" is included to provide a gap between the last digit and the unit
	column_byte_begin = (DIGIT_WIDTH_BYTE*3) + 1;
	column_byte_end = column_byte_begin + UNIT_WIDTH_BYTE;

	if(SelectedUnit == UNIT_MICROAMPS)
		LCD_Unit = LCD_microAmps;
	else if(SelectedUnit == UNIT_MILLIAMPS)
		LCD_Unit = LCD_milliAmps;
	else if(SelectedUnit == UNIT_MILLIVOLTS)
		LCD_Unit = LCD_milliVolts;
	else if(SelectedUnit == UNIT_VOLTS)
		LCD_Unit = LCD_Volts;
	else if(SelectedUnit == UNIT_HERTZ)
		LCD_Unit = LCD_Hz;

	/*here the LCD display data array is loaded with the bytes
	 from the appropriate pixel data array generated by Scilab*/
	for(row = 0; row < UNIT_HEIGHT_PIX; row++)
	{
		for(column_byte = column_byte_begin; column_byte < column_byte_end; column_byte++)
		{
			LCDDisplayData[row][column_byte] = *LCD_Unit;
			LCD_Unit++;
		}
	}

	//wait until the SPI state variable indicates that the bus is available for a new transfer
	while(LCDTxState != IDLE);

	//the SPI state machine needs to know the first and last lines to be updated
	LCDLineStart = UNIT_BEGIN_LINE;
	LCDLineStop = UNIT_BEGIN_LINE + UNIT_HEIGHT_PIX;
	UpdateLCD();

}

void SetDecimalPoint(unsigned char DecimalPointStatus)
{
	unsigned char row;
	unsigned char column_byte;

	//the decimal point can only be located after the first digit
	column_byte = DIGIT_WIDTH_BYTE - 1;

	/*the decimal point requires an area of 4 pixels by 4 pixels,
	 but the displayed shape is rounded because the top and bottom
	 lines have 2 horizontal black pixels and the 2 middle lines have
	 4 horizontal black pixels*/

	/*note the use of bitwise AND and OR operations here: bitwise
	 operations are needed because the decimal point does not cover
	 8 horizontal pixels (i.e., one horizontal byte), and AND and OR are used
	 to ensure that other pixels in the byte are not altered*/

	if(DecimalPointStatus == DEC_POINT_OFF)
	{
		row = (DIGIT_HEIGHT_PIX - DEC_POINT_HEIGHT);
		LCDDisplayData[row][column_byte] |= BIT0;
		LCDDisplayData[row][column_byte + 1]  |= BIT7;
		row++;
		LCDDisplayData[row][column_byte] |= (BIT1|BIT0);
		LCDDisplayData[row][column_byte + 1] |= (BIT7|BIT6);
		row++;
		LCDDisplayData[row][column_byte] |= (BIT1|BIT0);
		LCDDisplayData[row][column_byte + 1]  |= (BIT7|BIT6);
		row++;
		LCDDisplayData[row][column_byte] |= BIT0;
		LCDDisplayData[row][column_byte + 1]  |= BIT7;
	}

	if(DecimalPointStatus == DEC_POINT_ON)
	{
		row = (DIGIT_HEIGHT_PIX - DEC_POINT_HEIGHT);
		LCDDisplayData[row][column_byte] &= ~BIT0;
		LCDDisplayData[row][column_byte + 1]  &= ~BIT7;
		row++;
		LCDDisplayData[row][column_byte] &= ~(BIT1|BIT0);
		LCDDisplayData[row][column_byte + 1] &= ~(BIT7|BIT6);
		row++;
		LCDDisplayData[row][column_byte] &= ~(BIT1|BIT0);
		LCDDisplayData[row][column_byte + 1] &= ~(BIT7|BIT6);
		row++;
		LCDDisplayData[row][column_byte] &= ~BIT0;
		LCDDisplayData[row][column_byte + 1]  &= ~BIT7;
	}

	//wait until the SPI state variable indicates that the bus is available for a new transfer
	while(LCDTxState != IDLE);

	//the SPI state machine needs to know the first and last lines to be updated
	LCDLineStart = DEC_PNT_LINE_BEGIN;
	LCDLineStop = DEC_PNT_LINE_END;
	UpdateLCD();
}

void ConvertMeasurementandDisplay(unsigned char VoltageorCurrentorFreq, unsigned int MeasuredValue)
{
	unsigned char remainder;

	//this function cannot process numbers longer than 4 digits
	if(MeasuredValue > 9999)
		return;

	/*if the measured value is longer than 3 digits,
	 we use volts (instead of millivolts) or
	 milliamps (instead microamps), and we divide
	 by 10 because only three digits can be displayed */
	else if(MeasuredValue > 999)
	{
		DecimalPointOnorOff = DEC_POINT_ON;	//decimal point is displayed
		MeasuredValue = MeasuredValue/10;
		if(VoltageorCurrentorFreq == VOLTAGE)
			SetLCDUnit(UNIT_VOLTS);
		else if(VoltageorCurrentorFreq == CURRENT)
			SetLCDUnit(UNIT_MILLIAMPS);
	}

	/*if the measured value is not longer than 3 digits,
	 we use millivolts or microamps*/
	else
	{
		DecimalPointOnorOff = DEC_POINT_OFF;	//decimal point is not displayed
		if(VoltageorCurrentorFreq == VOLTAGE)
			SetLCDUnit(UNIT_MILLIVOLTS);
		else if(VoltageorCurrentorFreq == CURRENT)
			SetLCDUnit(UNIT_MICROAMPS);
		else if(VoltageorCurrentorFreq == FREQUENCY)
			SetLCDUnit(UNIT_HERTZ);
	}

	/*the modulus operator is used to obtain the first digit, which
	 corresponds to the remainder that would result from dividing
	 by 10; we then twice divide the measured value by 10 and repeat
	 the modulus operation to obtain the remainders corresponding
	 to the next two digits*/
	remainder = MeasuredValue % 10;
	SetLCDDigit(DIGIT_POS_3, MatchDigittoArray(remainder));

	MeasuredValue = MeasuredValue/10;
	remainder = MeasuredValue % 10;
	SetLCDDigit(DIGIT_POS_2, MatchDigittoArray(remainder));

	MeasuredValue = MeasuredValue/10;
	remainder = MeasuredValue % 10;
	SetLCDDigit(DIGIT_POS_1, MatchDigittoArray(remainder));

	/*this statement must occur after the
	 calls to SetLCDDigit() so that the decimal point is not
	 overwritten by the pixel data for the digits*/
	SetDecimalPoint(DecimalPointOnorOff);
}

/*this function returns a pointer to the pixel data array
that corresponds to the value in InputDigit*/
unsigned char* MatchDigittoArray(unsigned char InputDigit)
{
	switch(InputDigit)
	{
		case 0: return LCD_Digit0;
		break;

		case 1: return LCD_Digit1;
		break;

		case 2: return LCD_Digit2;
		break;

		case 3: return LCD_Digit3;
		break;

		case 4: return LCD_Digit4;
		break;

		case 5: return LCD_Digit5;
		break;

		case 6: return LCD_Digit6;
		break;

		case 7: return LCD_Digit7;
		break;

		case 8: return LCD_Digit8;
		break;

		case 9: return LCD_Digit9;
		break;
	}

	/*if for some reason InputDigit is greater than 9,
	 we display an "X" character to indicate an error*/
	return LCD_Error;
}







